/***********************************************************
** evt2sac.c 
**
** Program to pull out seismic data from CD_ROM *.evt file
**
** Syntax:
**	evt2sac control_file
**
** where control_file is the name of the file with the controls
**
** If no control_file is specified, the program looks for the file:
**
**	evt2sac.pro
**
** Alan L. Jones, SUNY Binghamton, 607-755-2519
** jones@sunquakes.geol.binghamton.edu
** 08/15/90: Integrating with evt_subs.c
** 90.09.18:	Got hard-coded directories out of fglue2.F
**		Added version number when program is invoked.
** 90.09.25:	Add capability to mark phases on SAC plot
** 90.09.26:	Add ability to set kztime to event origin or beginning of trace
** 90.10.09:	Add capability for user to change parameters
** 90.10.26:	Make user hit CR after displaying banner
** 90.11.20:	Add capability of user to specify directory where jb tables
**		are stored and to specify the file extension.
** 90.12.10:	Take out scaling.  Leave results in counts.
** 91.03.11:	Fix PP.jb and SS.jb tables.  Add depth to SAC header
** 91.08.07:	Assign IZTYPE according to Origin Time or Begin Time.  Before
**		it was always set to Begin Time.  Fix bug in epoch time.
** 91.09.18:	Fix depth.  Was not being put in SAC header as advertixed on 91.03.11
**		Enlarge input text buffer from 80 bytes to 200 to handle long lines
**		in profile.
***********************************************************/

#include <stdio.h>
#include <ctype.h>
static char *banner[] = {
"                             EVT2SAC",
"                    Version 1.0, Level 91.09.18",
"",
"A Program to Extract Seismic Files from CD_ROM and Convert to SAC Format",
"",
"                     written by Science Horizons", 
"    modified by Alan Jones, Jeff Barker, Francis Wu, SUNY Binghamton",
"                     modifications sponsored by:",
"        Incorporated Research Institutions for Seismology (IRIS)",
"",
"(CR to continue)",
NULL};



/******************************
** The following global variables are defined in evt_subs.c
******************************/
extern float rmin, rmax;		/* min, max range */
extern float mmin, mmax;		/* min, max minutes of lag */
extern float min_before, min_after;	/* min before and after specified phase */
extern char initial_phase[];		/* Initial phase, e.g. "P"	*/
extern char final_phase[];		/* Final phase, e.g. "surface"	*/
extern float sw_velocity_1;		/* Surface wave velocity */
extern float sw_velocity_2;		/* Surface wave velocity */
extern char sac_prefix[];		/* Add to front of sac file name */
extern char sac_suffix[];		/* Add to end of sac file name */
extern char **mark_phases;		/* Phases to mark (time pick) */
extern int o_trace;			/* Is KZTIME set to beginning of trace? */
extern char ttc_ex[];			/* Travel time curve extension */
extern char ttc_dir[];			/* Travel time curve directory */
static int event_number;		/* Event number */


/**************************
** main()
**************************/
main(argc,argv)
int argc;
char **argv;
{
	/*************************
	**  	DATA
	*************************/
	int i, n, done, changed = 0;
	FILE *fp;
	char path[80];
	char buffer[200];
	char filename[16];
	char profile[20];	/* profile file name */
	char phase[5];
	char *p;
	char s[7][6];	/* To store phases */
	char **b;
	char ch;
	int year, month, day;
	static char *months[] = 
		{"jan","feb","mar","apr","may","jun",
		"jul","aug","sep","oct","nov","dec"};

	/************************
	** 	CODE
	************************/

	b = banner;
	while(*b) printf("%s\n",*b++);
	fgets(buffer,200,stdin);

	/*** Set defaults ***/
	for(i=0;i<7;i++) strcpy(s[i],"");
	strcpy(path,"");
	strcpy(filename,"");
	mark_phases = (char **) malloc(sizeof(char *));
	mark_phases[0] = NULL;
	year = 1980;
	month = 1;
	day = 1;
	rmin = 0;
	rmax = 180;
	mmin = 0;
	mmax = 360;
	o_trace = 0;
	min_before = min_after = 0;
	strcpy(ttc_ex,".jb");
	strcpy(ttc_dir,"");
	strcpy(initial_phase,"");
	strcpy(final_phase,"");
	sw_velocity_1 = sw_velocity_2 = 4.2;
	if(argc == 2) strcpy(profile,argv[1]);
	else strcpy(profile,"evt2sac.pro");
	fp = fopen(profile,"r");
	if(fp == NULL)
	{
		printf(
"\nSYNTAX:\n\nevt2sac filename\n\nwhere filename is name of control file.\n\n");
		exit(1);
	}

	while(fgets(buffer,80,fp))
	{
		if('#' == *buffer) continue;	/* Comment */
		if(' ' == *buffer) continue;	/* blank in first space */
		if(strlen(buffer) <= 1) continue;	/* empty line */
		/* Turn command into upper case */
		for(p = buffer;*p && *p != ':' ; p++) 
		{
			if(islower(*p)) *p = toupper(*p);
		}
		if(sscanf(buffer,"PATH: %s",path));
		else if(sscanf(buffer,"YEAR: %u",&year));
		else if(sscanf(buffer,"MONTH: %u",&month));
		else if(sscanf(buffer,"DAY: %u",&day));
		else if(sscanf(buffer,"MAXIMUM RANGE: %f",&rmax));
		else if(sscanf(buffer,"MINIMUM RANGE: %f",&rmin));
		else if(sscanf(buffer,"MAXIMUM MINUTES: %f",&mmax));
		else if(sscanf(buffer,"MINIMUM MINUTES: %f",&mmin));
		else if(sscanf(buffer,"MINUTES BEFORE: %f",&min_before));
		else if(sscanf(buffer,"MINUTES AFTER: %f",&min_after));
		else if(sscanf(buffer,"INITIAL PHASE: %s",initial_phase));
		else if(sscanf(buffer,"FINAL PHASE: %s",final_phase));
		else if(sscanf(buffer,"SURFACE WAVE VELOCITY 1: %f",&sw_velocity_1));
		else if(sscanf(buffer,"SURFACE WAVE VELOCITY 2: %f",&sw_velocity_2));
		else if(sscanf(buffer,"SAC FILE NAME PREFIX: %s",sac_prefix));
		else if(sscanf(buffer,"SAC FILE NAME SUFFIX: %s",sac_suffix));
		else if(sscanf(buffer,"EVENT NUMBER: %u",&event_number));
		else if(sscanf(buffer,"PHASE FILE EXTENSION: %s",ttc_ex));
		else if(sscanf(buffer,"PHASE FILE DIRECTORY: %s",ttc_dir));
		else if(sscanf(buffer,"MARK PHASE: %s",phase)) {
			/* find last phase marked */
			for(i=0;mark_phases[i];i++) ;
			mark_phases = (char **) realloc(mark_phases,
				(i+2)*sizeof(char **));
			mark_phases[i] = (char *) malloc(5);
			strcpy(mark_phases[i++],phase);
			mark_phases[i] = NULL;
		}
		else if(!strncmp("SET KZTIME TO START OF TRACE",buffer,28)) o_trace = 1;
		else {
			printf("\nUnknown command: %s\n",buffer);
			exit(1);
		}
	}
	fclose(fp);

	/************************************************************
	** 90:10:09: Print out parameters and allow user to change
	************************************************************/
	done = 0;
	do {
		i = 0;
		printf("\nThe current value of the parameters:\n\n");
		printf("%2u: PATH  . . . . . . . . . . . . . . %s\n",++i,path);
		printf("%2u: YEAR  . . . . . . . . . . . . . . %u\n",++i,year);
		printf("%2u: MONTH . . . . . . . . . . . . . . %u\n",++i,month);
		printf("%2u: DAY . . . . . . . . . . . . . . . %u\n",++i,day);
		printf("%2u: MAXIMUM RANGE . . . . . . . . . . %.1f degrees\n",++i,rmax);
		printf("%2u: MINIMUM RANGE . . . . . . . . . . %.1f degrees\n",++i,rmin);
		printf("%2u: MAXIMUM MINUTES . . . . . . . . . %.1f\n",++i,mmax);
		printf("%2u: MINIMUM MINUTES . . . . . . . . . %.1f\n",++i,mmin);
		printf("%2u: MINUTES BEFORE  . . . . . . . . . %.2f\n",++i,min_before);
		printf("%2u: MINUTES AFTER . . . . . . . . . . %.2f\n",++i,min_after);
		printf("%2u: INITIAL PHASE . . . . . . . . . . %s\n",++i,initial_phase);
		printf("%2u: FINAL PHASE . . . . . . . . . . . %s\n",++i,final_phase);
		printf("%2u: SURFACE WAVE VELOCITY 1 . . . . . %.2f km/sec\n",++i,sw_velocity_1);
		printf("%2u: SURFACE WAVE VELOCITY 2 . . . . . %.2f km/sec\n",++i,sw_velocity_2);
		printf("%2u: SAC FILE NAME PREFIX  . . . . . . %s\n",++i,sac_prefix);
		printf("%2u: SAC FILE NAME SUFFIX  . . . . . . %s\n",++i,sac_suffix);
		printf("%2u: EVENT NUMBER  . . . . . . . . . . %u\n",++i,event_number);
		printf("%2u: SET KZTIME TO START OF TRACE  . . %u (1: true, 0: false)\n",++i,o_trace);
		printf("%2u: PHASE FILE EXTENSION  . . . . . . %s\n",++i,ttc_ex);
		printf("%2u: PHASE FILE DIRECTORY  . . . . . . %s\n",++i,ttc_dir);
		printf("%2u: MARK PHASE(s) . . . . . . . . . . ",++i);
		b = mark_phases;
		while(*b) {
			printf("%s ",*b++);
		}
		printf("\n\n");
		printf("To change a parameter, enter the number followed by the\n");
		printf("new value(s).  E.g: to change Maximum range to 62 degrees, enter:\n");
		printf("\n5 62\n\(CR to continue)\n");
		fgets(buffer,80,stdin);
		buffer[strlen(buffer)-1] = 0;	/* Get rid of new line character */
		if(strlen(buffer) == 0) break;
		changed = 1;
		sscanf(buffer,"%u",&n);
		if(n < 0 || n > i) {
			printf("Number out of range.\n");
		} else {
			switch(n) {
			case 1: if(!sscanf(buffer,"%*u %s",path)) 
				strcpy(path,"");;
			break;
			case 2: sscanf(buffer,"%*u %u",&year);
			break;
			case 3: sscanf(buffer,"%*u %u",&month);
			break;
			case 4: sscanf(buffer,"%*u %u",&day);
			break;
			case 5: sscanf(buffer,"%*u %f",&rmax);
			break;
			case 6: sscanf(buffer,"%*u %f",&rmin);
			break;
			case 7: sscanf(buffer,"%*u %f",&mmax);
			break;
			case 8: sscanf(buffer,"%*u %f",&mmin);
			break;
			case 9: sscanf(buffer,"%*u %f",&min_before);
			break;
			case 10: sscanf(buffer,"%*u %f",&min_after);
			break;
			case 11: if(!sscanf(buffer,"%*u %s",initial_phase))
				 strcpy(initial_phase,"");
			break;
			case 12: if(!sscanf(buffer,"%*u %s",final_phase))
				strcpy(final_phase,"");
			break;
			case 13: sscanf(buffer,"%*u %f",&sw_velocity_1);
			break;
			case 14: sscanf(buffer,"%*u %f",&sw_velocity_2);
			break;
			case 15: if(!sscanf(buffer,"%*u %s",sac_prefix))
				strcpy(sac_prefix,"");;
			break;
			case 16: if(!sscanf(buffer,"%*u %s",sac_suffix))
				strcpy(sac_suffix,"");
			break;
			case 17: sscanf(buffer,"%*u %u",&event_number);
			break;
			case 18: sscanf(buffer,"%*u %u",&o_trace);
			break;
			case 19: sscanf(buffer,"%*u %s",ttc_ex);
			break;
			case 20: sscanf(buffer,"%*u %s",ttc_dir);
			break;
			case 21: n=sscanf(buffer,"%*u %s %s %s %s %s %s %s",
				s[0],s[1],s[2],s[3],s[4],s[5],s[6]);
				/* Free up existing phases */
				if(mark_phases) {
					i = 0;
					while(mark_phases[i]) {
						free(mark_phases[i++]);
					}
					free(mark_phases);
				}
				mark_phases = (char **) malloc((n+1)*sizeof(char **));
				for(i=0;i<n;i++) {
					mark_phases[i] = (char *) malloc(1+strlen(s[i]));
					strcpy(mark_phases[i],s[i]);
				}
				mark_phases[i] = NULL;
			}
		}
	} while(!done);
	if(changed) {
		printf("\nDo you wish to save this profile?\n");
		if(yes()) {
			printf("Enter profile name.  (Default = %s.)\n",profile);
			fgets(buffer,20,stdin);
			buffer[strlen(buffer)-1] = 0;
			if(strlen(buffer)) strcpy(profile,buffer);
			fp = fopen(profile,"w");
			if(fp == NULL) {
				fprintf("\nProblem creating profile: %s.\n",profile);
				exit(1);
			} else {
				fprintf(fp,"# Profile created by evt2sac\n");
				fprintf(fp,"PATH:                    %s\n",path);
				fprintf(fp,"YEAR:                    %u\n",year);
				fprintf(fp,"MONTH:                   %u\n",month);
				fprintf(fp,"DAY:                     %u\n",day);
				fprintf(fp,"MAXIMUM RANGE:           %f\n",rmax);
				fprintf(fp,"MINIMUM RANGE:           %f\n",rmin);
				fprintf(fp,"MAXIMUM MINUTES:         %f\n",mmax);
				fprintf(fp,"MINIMUM MINUTES:         %f\n",mmin);
				fprintf(fp,"MINUTES BEFORE:          %f\n",min_before);
				fprintf(fp,"MINUTES AFTER:           %f\n",min_after);
				fprintf(fp,"INITIAL PHASE:           %s\n",initial_phase);
				fprintf(fp,"FINAL PHASE:             %s\n",final_phase);
				fprintf(fp,"SURFACE WAVE VELOCITY 1: %f\n",sw_velocity_1);
				fprintf(fp,"SURFACE WAVE VELOCITY 2: %f\n",sw_velocity_2);
				fprintf(fp,"SAC FILE NAME PREFIX:    %s\n",sac_prefix);
				fprintf(fp,"SAC FILE NAME SUFFIX:    %s\n",sac_suffix);
				fprintf(fp,"EVENT NUMBER:            %u\n",event_number);
				if(o_trace)
					fprintf(fp,"SET KZTIME TO START OF TRACE:\n");
				b = mark_phases;
				while(*b) {	
					fprintf(fp,"MARK PHASE:             %s\n",*b++);
				}
				fclose(fp);
			}
		}		
	}
	
	/*** Create file name ***/
	sprintf(filename,"%2u%03u00",year%100,julian(year,month,day));

	/*** Create path name ***/
	i = strlen(path);
	if(i)
	  if(path[i-1] != '/') strcat(path,"/");
	sprintf(buffer,"%s%4u/%s",path,year,months[month-1]);
	
	/****************************
	** Process the specified *.evt file
	****************************/

	proc_evt(filename,buffer);
	
}
/*************************
** write_tty(cmd)
**
** Program for writing to screen
*************************/
write_tty(cmd)
char *cmd;
{
	printf(cmd);
}
/*************************
** which_evt()
** Program to select which event
** Normally the events are displayed at this point and the user
** selects which one.  However, since we used our *.inx file, the
** user has already picked the event.  We just have to find which
** one it is.  We'll cheat.  That is, we'll just see how many events
** have the selected *.evt file and find which number is the one
** selected.
*************************/
int which_evt(p)
char *p;	/* event log */
{
	int i, j;

	/*** If user has specified which event, just return that number ***/
	if(event_number) return event_number;

	for(i=0, j=0;i<25;i++, p +=81)
	{
		if(p[5] == '1')
		{
			j++;
			/* Have hit a year.  Have to fix in year 2000 */
			printf("%3u %s\n",j,p);
		}
		else if(!j) printf("    %s\n",p); /* Don't print bottom lines */
	}
	printf("Which event?\n");
	scanf("%u",&i);
	return i;
}

/******************
** yes()
**
** Get user response.  If 'y' or 'Y' return true;
******************/
int yes()
{
	char buffer[20], c;
	fgets(buffer,20,stdin);
	c = *buffer;
	if(c == 'y' || c == 'Y') return 1;
	else return 0;
}

	
