/**************************************************************************
** evt2_subs.c
** Sub-routines to do the bulk of the work for converting a  CD_ROM *.evt file
** to SAC files.
**
** Alan L. Jones, June 19, 1990
** Took original evt2sac.c which was a modified version of evt_sac.c.
** Modifications by J. Barker made the program more interactive by
** asking for range and time span.
**
** 90.07.02:	Make more system independent so can move to other platforms
**		such as a PC
** 90.08.20:	Take Sunview stuff out of this module and put in sun_subs.c
** 90.08.21:	Put evt2sac.c into evt_subs.c so the routines can be called
** 		in a manual mode or by Sunview programs
** 90.08.27:	Completely re-write logic.  Found old evt2sac program was
**		assuming a short period seismogram had only one channel.
**		Process short, intermediate, and long the same way.
** 90.08.28:	Got it working so that program gets multiplexed short period records
**		and hooks them together properly.
** 90.09.12:	-1 Version 0.1
**		Consolidate into one sub-directory.  Get rid of lib and include
**		sub-directories.  lib contained compiled time computations for
**		which we had no source so code could not be compiled on a
**		workstation with a different architecture.  The htoe()
**		(hour-to-epoc) and "day of year" computations which the csstime
**		provided were replaced with our own computations.
** 90.09.13	Fix data_break routine.  It concluded there was a break if
**		the time difference was 0 to 10 milliseconds.  If it was
**		e.g. -1 milliseconds, it concluded there was a break.  Made
**		the criterion -5 to +5 milliseconds
** 90.09.19	Extended for other phases after generating files pP.DAT, sS.DAT
**		PP.DAT, SS.DAT in addition to the files we already have: P.DAT,
**		S.DAT
** 90.09.20	Changed names of travel time files from *.DAT to *.ttc and
**		changed the data in them from tenths of seconds to seconds.
**		Change NZTIME from event time to beginning-of-trace time.
** 90.09.24	Change names of *.ttc to *.jb
** 90.09.25:	Implement marking of phases
** 90.09.26:	Add capability for user to select reference time to be
**		either event time or beginning of trace.
** 90.11.14:	Add capability for user to set environment variable EVT2SAC
**		to locate *.jb files
** 90.11.20:	At Tom Owens' suggestion, allow user to specify location
**		and file extension of travel time curves.
** 91.03.11:	PP and SS travel times cont computed correctly.  Fixed
**		Add depth to SAC header (EVDP)
** 91.08.07:	Set IZTYPE to IB (Begin Time) or IO (Event Time Origin)
**		Had not been setting it before.
**
****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <sys/file.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <math.h>
#include <ctype.h>
#include "ndt_logs.h"
#include "sac_hdr.h"
#include "trim.h"

/*** Can't find header for getenv() so defining here to avoid warning ***/
char *getenv();

/*** prototypes ***/
static int sift_rt(struct event_log *el,
	struct data_header *dh,
	struct station_log *sl,
	struct sac_hdr *sh,
	int nev,
	float rmin,
	float rmax,
	float *tlag);


char prefix[20];
char outfile[50];
char tblock[2000];
char cards[25][81];
int dblock[1000];
int ablock[3][400];

/*** Variables from original evt2sac.c program ***/
int ifd;	/* File descriptor for CD-ROM file */
static int nev = 1;

/* File pointers for other files */
FILE *hfp;	/* prefix.hdr file pointer */
FILE *dfp;	/* prefix.dat file pointer */

struct file_header file_header;
struct tape_log tape_log;
struct event_log event_log;
struct station_log station_log;
struct data_log1 data_log1;
struct data_header data_header;

int sac_seq = 0;
int cdver = 2;		/**********************************************
			** CD version.  If cdver == 1, then the bytes
			** on the CD-ROM are in VAX order which means
			** that each group of 4 bytes must be reversed.
			** We will assume they are not swapped and change
			** cdver to 1 if we find they must be.
			**********************************************/
struct sac_hdr sac_hdr[3];
float sac_data[3][200000];
char file_name[30];
float i_time;
float f_time;
int **mark_ttc;	/* array of travel time curves for marking phases */


/****************************
** Following variables are externed in main routine
****************************/
float rmin,rmax;	/* max, min range in degrees */
char initial_phase[10];	/* initial phase, e.g. "pP" */
char final_phase[10];	/* final phase */
float min_before, min_after;	/* Number of minutes before */
			/* and after give phase */
float sw_velocity_1;	/* Surface wave velocity */
float sw_velocity_2;	/* Surface wave velocity */
float mmin, mmax;		/* Minimum and maximum minutes */
int initial_ttc[181];	/* travel time curve to initial phase */
int final_ttc[181];	/* travel time curve to final phase */
char sac_prefix[10];	/* To put on beginning of file name */
char sac_suffix[10];	/* to put on end of file name */
char **mark_phases;	/* Names of phases to mark */
int **phase_ttc;	/* Travel time curves.  One for each marked phase */
int o_trace;		/* Is KZTIME to be set to beginning of trace? */
char ttc_ex[10];	/* Travel time curve extension */
char ttc_dir[60];	/* Travel time curve directory */

/****************************************
** Following variables are used to put values into SAC header
****************************************/

static char periods[] = {'s','l','i'};		/* short, long, intermediate */
static char directions[] = {'z','n','e'};	/* north, east, vertical */
static float cmpaz[] = { 00.0, 00.0, 90.0 };	
static float cmpia[] = { 00.0, 90.0, 90.0};

/**************************
** lower()
**
** Change a string to lower case
**************************/
void lower(char *p)
//char *p;
{
	for(;*p;p++) if(isupper(*p)) *p = tolower(*p);
}
#if 0
/**************************
** strstr()
**
** Find a string in a string
***************************/
char *strstr(str1,str2)
char *str1;
char *str2;
{
	int len;
	len = strlen(str2);
	for(;*str1;str1++)
		if(!strncmp(str1,str2,len))
			return(str1);
	return NULL;
}
#endif
/**************************
** htoe()
**
** Computes epoch time in seconds from January 1, 1969
** We pick 1/1/69 since we can then multiply the year times 365.25 and
** the lengths of the years will come out right when the days are
** truncated to an integer.  That is, 1969-1971 will have 365 days
** and 1972 366 days.
** 
** NOTE: does not include corrections for leap seconds on 6/30 or 12/30
**
** INPUT: pointer to a structure "date_time" defined in ndt_logs.h:
** struct date_time{
**        double epoch;
**        long date;
**        int year;
**        int month;
**        char mname[4];
**        int day;
**        int doy;
**        int hour;
**        int minute;
**        float second;
** };
**************************/
void htoe(pdt)
struct date_time *pdt;
{
	pdt->epoch = (double)pdt->second + 
		60*(pdt->minute+
		60*(pdt->hour+
		24*(pdt->doy+(int)((pdt->year-1969)*365.25))));
}
/**************************
** inter_ttc(ttc,arc)
**
** Interpolate for travel time curve for any arc (0 to 180)
**************************/
float inter_ttc(ttc,arc)
int *ttc;
float arc;
{
	int iarc;
	float time;
	iarc = arc;
	time = ttc[iarc];
	time += (ttc[iarc+1] - time)*(arc - iarc);
	return time;
}

/**************************
** ymd()
**
** Gives year, month, and day if given number of days since 1/1/1969
** 8/7/91: David Salzberg found bug for case of month = December.  Fixed.
**************************/
void ymd(eday,pyear,pmonth,pday)
int eday;
int *pyear;
int *pmonth;
int *pday;
{
	static int months[2][13] = {
		0,31,59,90,120,151,181,212,243,273,304,334,365,
		0,31,60,91,121,152,182,213,244,274,305,335,366
		};
	int i, j;
	int d;
	d = 1969 +((float)eday)/365.25;
	*pday = eday - (int)((float)(d-1969)*365.25);
	if(*pday == 0) {
		*pday = 366;
		d--;
	}
	if(0 == (d%400)) i = 1;
	else if (0 == (d%100)) i = 0;
	else if (0 == (d%4)) i = 1;
	else i = 0;
	for(j=1;*pday>months[i][j];j++);
	*pday -= months[i][j-1];
	*pmonth = j;
}

/************************
** etoh()
**
** Does reverse of htoe().  That is, it computes new day, year, etc.
** based on epoch time
************************/
void etoh(pdt)
struct date_time *pdt;
{
	float f;
	long  a;
	a = pdt->epoch;
	f = pdt->epoch - a;	/* Fractional part of second */
	pdt->second = a%60;
	pdt->second += f;
	a /= 60;
	pdt->minute = a%60;
	a /= 60;
	pdt->hour = a%24;
	a /= 24;
	/* 'a' is now number of days since 1/1/1969 */
	ymd(a,&pdt->year,&pdt->month,&pdt->day);
	pdt->doy = julian(pdt->year,pdt->month,pdt->day);
}
/**************************
** julian(year,month,day)
** 
** Compute julian day
***************************/
int julian(year,month,day)
int year, month, day;
{
	int ly;		/* ly = 1 if a leapyear, ly = 0 otherwise */
	int d = 0;	/* Julian day */
	int i;		/* counter */
	static int months[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

	month--;	/* change month to origin 0 */
	ly = 0;
	if((year%400) == 0) ly = 1;
	else if(year%100 == 0) ly = 0;
	else if(year%4 == 0) ly = 1;
	else ly = 0;
	months[1] = (ly) ? 29: 28;
	for(i=0;i<month;i++) d += months[i];
	d += day;
	return d;
}

/**************************
** get_ttc()
**
** Get travel time curve
**
** Returns:
**	0	Travel time curve created
**	1	Unsupported phase requested.  ttc set to 0.
**	2	Couldn't find ttc file.  ttc set to 0.
**************************/
static int get_ttc(ttc,phase,depth,velocity)
int *ttc;	/* An array with 181 elements which will receive the data */
		/* 9/5/90: These times are stored as integer seconds */
char *phase;	/* Which phase is requested.  At present only support "S" */
		/* "P", and "surface"	*/
float depth;	/* Depth of event in km */
float velocity;	/* Velocity of surface wave in km/sec. */
{
	char filename[80];
	FILE *fp;
	int i, h;
	int *p;
	char *env;	/* Points to environment variable */
	int times[14];	/* Travel time curve for each depth */
	
	/*** CODE ***/
	p = ttc;
	strcpy(filename,"");
	for(i=0;i<=180;i++) *p ++= 0;
	p = ttc;
	if(!strcmp("surface",phase))
	{
		/*** Convert velocity from km/sec to degrees/sec ***/
		velocity = velocity*180./(6371*3.14159);
		for(i=0;i<=180;i++) *p++ = .5 + i/velocity;
	} else {
		if(strlen(ttc_dir)) 
		{
			strcpy(filename,ttc_dir);
			strcat(filename,"/");
		}
		else strcpy(filename,"");
		strcat(filename,phase);
		strcat(filename,ttc_ex);
		fp = fopen(filename,"rb");	
		if(fp == NULL) {
			printf("\nCan't find file: %s\n\n",filename);
			exit(1);
                }
		if(depth < 33.) h = 0;
		else
			/* 100 is needed since jb tables have h = 0.01, 0.02 etc. */
			h = 1.5 + 100*(depth - 33)/6338;
		i = 0;
		/* Times read from file are in second s*/
		while(fread(times,sizeof(int),14,fp))
		{
			ttc[i++] = times[h];
			if(i>180) break;
		}
		fclose(fp);
	}		
}
/**************************
** write_sac()
**
** write SAC file.
**************************/
static void write_sac()
{
	/***************************
	**	DATA
	***************************/
	int i,j,k,p;
	int n_chan;		/* Number of channels */
	int i_chan;		/* Channel counter */
	int iok,first;		/* flags */
	int *pi;		/* pointer to an integer */
	int n_blk;		/* number of blocks of data to process */
	int n_blks[3];		/* number of blocks for short, long, intermediate */
	int per[3];		/* 0: short period, 1: long period, 2: intermediate */
	float tlag = 0.;
	float depth;
	int smin, smax;		/* minimum and maximum seconds */
	char string[80];
	int direction;	/* direction = 0: north, 1: east, 2: up */
	int period;	/* period = 0: short, 1: intermediate, 2: long */

	/************************
	**	CODE
	************************/

	smin = mmin*60;		/* minimum time in seconds */
	smax = mmax*60;		/* maximum time in seconds */

	/*
	**   All done with header info. Now loop over stations, extracting
 	**   the station log and any data logs. Collect the station log and
	**   data logs info in prefix.dat .
	*/
	
	strcpy(outfile,prefix);
	strcat(outfile,".dat");
	if( (dfp = fopen(outfile,"w")) == NULL ){
		fprintf(stderr,"Can't open:%s\n",outfile);
		perror("evt_css");
		exit(3);
	}
	depth = event_log.edepth[nev-1];
	/***************************************
	** Compute data for travel time curves
	** The function get_ttc() determines which phase user wants
	** and fills the ttc array with the number of seconds for each
	** degree from 0 to 180.  Then this array is adjusted according
	** to the min and max times and for the time specified that the
	** user wants to start before and end after the arrivals specified.
	***************************************/
	if(strlen(initial_phase))
		get_ttc(initial_ttc,initial_phase,depth,sw_velocity_1);

	/*** adjust for user specified minimum time (in seconds ) ***/
	j = min_before*60;
	for(i=0, pi = initial_ttc;i<=180;i++, pi++)
	{
		if(*pi == 0) *pi = smin;
		else *pi -= j;
		if(smin > *pi) *pi = smin;
	}
	if(strlen(final_phase))
		get_ttc(final_ttc,final_phase,depth,sw_velocity_2);

	/*** adjust for user specified maximum time */
	j = min_after*60;
	
	for(i=0, pi = final_ttc;i<=180;i++, pi++)
	{
		if(*pi == 0) *pi = smax;
		else *pi += j;
		if(smax < *pi) *pi = smax;
	}

	/***************************************
	** Get travel time curves for marked phases
	***************************************/
	phase_ttc = (int **)malloc(sizeof(int **));
	phase_ttc[0] = NULL;
	i = 0;
	while(mark_phases[i]) {	
		phase_ttc = (int **)realloc(phase_ttc,(i+2)*sizeof(int **));
		phase_ttc[i] = (int *)malloc(sizeof(int)*181);
		get_ttc(phase_ttc[i],mark_phases[i],depth,sw_velocity_1);
		i++;
		phase_ttc[i] = NULL;
	}
	write_tty("\r\n");	/* Start TTY output on new line */
	for( k = 0 ; k < file_header.nsta ; k++ ){
		sprintf(string,"   *** Station %-4s ***\r\n",tape_log.sta[k]);
		write_tty(string);
		/*
 		**   Read and interpret the STATION LOG.
		*/
		read_block(ifd,tblock);
		line_split(tblock,cards);
		crack_station_log(cards,&station_log);
		/* 
		printf("STATION: %s\n",station_log.sta);
		printf("%s\n",cards[0]);
		getchar();
		*/
		for( i = 0 ; i < 25 ; i++ ) fprintf(dfp,"%s\n",cards[i]);

		/*********************************************
		** Determine if the next data records are sp, lp, or ip.
		*********************************************/

		n_blks[0] = file_header.nspblk[k];
		n_blks[1] = file_header.nlpblk[k];
		n_blks[2] = file_header.nipblk[k];
		for(p = 0;p<3;p++) {	/* Cycle through sp, lp, ip */
		  period = p;
		  n_blk = n_blks[p];
		  if(n_blk){
		    /***  Get three blocks of data log. ***/
		    read_block(ifd,tblock);
		    line_split(tblock,cards);
		    crack_data_log1(cards,&data_log1);
		    for( i = 0 ; i < 25 ; i++ )fprintf(dfp,"%s\n",cards[i]);
		    read_block(ifd,tblock);
		    line_split(tblock,cards);
		    for( i = 0 ; i < 25 ; i++ )fprintf(dfp,"%s\n",cards[i]);
		    read_block(ifd,tblock);
		    line_split(tblock,cards);
		    for( i = 0 ; i < 25 ; i++ )fprintf(dfp,"%s\n",cards[i]);
		    /***  Loop over data blocks.  ***/
		    sac_seq = 0;
		    first=1;
		    for( i = 0 ; i < (n_blk -3) ; i++ ){
		      read_block(ifd,tblock);
		      crack_data_header(tblock,&data_header);
		      n_chan = data_header.nchan;
		      iok = sift_rt(&event_log,&data_header,&station_log,
                                          &sac_hdr[0],nev,rmin,rmax,&tlag);
		      if(iok){
			reformat_data(tblock,dblock,data_header.nsamp,data_header.fmtid);
			if(n_chan == 3)
			  demux(dblock,ablock[0],ablock[1],ablock[2],data_header.nsamp);
			if(first){
			  for(i_chan = 0; i_chan < n_chan; i_chan++)
			      init_sac_hdr(&sac_hdr[i_chan],&data_header,
					 &station_log,&data_log1,&event_log,&tlag,i_chan,period);
			}
			else{
			  if(data_break(&sac_hdr[0],&data_header) ){
			    for(i_chan=0;i_chan<n_chan;i_chan++)
			      flush_sac_file(&sac_hdr[i_chan],sac_data[i_chan]);
			    sac_seq++;
			    for(i_chan=0;i_chan<n_chan;i_chan++)
			      init_sac_hdr(&sac_hdr[i_chan],&data_header,
                                     &station_log,&data_log1,&event_log,&tlag,i_chan,period);
			  }
			}
			if(n_chan == 1)
			  sac_conv(sac_data[0],&sac_hdr[0],dblock,data_header.nsamp);
			else
			  for(i_chan=0;i_chan<n_chan;i_chan++)
				 sac_conv(sac_data[i_chan],&sac_hdr[i_chan],
                                            ablock[i_chan],data_header.nsamp / 3);
			  first=0;
		      }
		      else{
		        if(!first){
			  for(i_chan=0;i_chan<n_chan;i_chan++)
			      flush_sac_file(&sac_hdr[i_chan],sac_data[i_chan]);
			  sac_seq++;
			  first=1;
			}
		      }
		    }
		    if(iok){
		      if(!first){
		        for(i_chan=0;i_chan<n_chan;i_chan++)
				flush_sac_file(&sac_hdr[i_chan],sac_data[i_chan]);
		      }
		    }
		  }
		}
	}

	fclose(dfp);
	close(ifd);
}

/*
**  If a gap larger than a 10 milliseconds exists, there is a break in the data.
*/
int data_break(sh,dh)
struct sac_hdr *sh;
struct data_header *dh;
{
        double nextepoch;
        int timediff;
        struct date_time dt;

        dt.year = sh->nzyear;
        dt.doy  = sh->nzjday;
        dt.date = dt.year * 1000 + dt.doy;
        dt.hour = sh->nzhour;
        dt.minute= sh->nzmin;
        dt.second= sh->nzsec + ((double)sh->nzmsec / 1000.);
        htoe(&dt);

        nextepoch = dt.epoch + sh->b + ((double)sh->npts * sh->delta);

	/****************************************************************
	** ALJ: 9/13/90: put in "0.5+" so we get zero difference on any
	** time difference between -5 ms and +5 ms.  Therefore, time
	** ALJ: 9/14/90: change to produce a difference if more than
	** 10% of delta 
	****************************************************************/
        timediff = 0.5+(nextepoch - dh->dt.epoch) * 5./sh->delta;

        return( timediff );
}

/*
**  Inititialize a brand new wfdisc index record based on the info in the
**  station log, data log 1, and the first data header.
**
** 08/27/90: ALJ Add event log to input parameters so we can set ztime to event origin time
**		rather than beginning of record time.
*/
int init_sac_hdr(sh,dh,sl,dl,el,tlag,cmp,period)
struct sac_hdr *sh;
struct data_header *dh;
struct station_log *sl;
struct data_log1 *dl;
struct event_log *el;
float *tlag;
int cmp;		/* Channel number */
int period;		/* 0: short period, 1: intermediate period, 2: long period */
{
        float find_calib();
	struct date_time *dt;

	dt = &el->edt[nev-1];
        sh->npts        = 0;
        sh->b           = *tlag;
        sh->e           = *tlag;
        sh->delta       = 1.0 / dh->smprat;
        sh->scale       = find_calib(cmp,dl);
	/* Set KZTIME to event time */
        sh->nzyear      = dt->year;
        sh->nzjday      = dt->doy;
        sh->nzhour      = dt->hour;
        sh->nzmin       = dt->minute;
        sh->nzsec       = dt->second;
        sh->nzmsec      = (int)(1000. * dt->second) % 1000;
        sh->iinst       = 0;
        strcpy(sh->kstnm,sl->sta);
        sh->stla        = sl->lat;
        sh->stlo        = sl->lon;
        sh->stel        = sl->elev;
	sprintf(sh->kcmpnm,"%c%c",periods[period],directions[cmp]);
        sh->cmpaz       = cmpaz[cmp];
        sh->cmpinc      = cmpia[cmp];
        sh->evdp	= event_log.edepth[nev-1];
	/* sh->o = el->edt[ev].epoch - dh->dt.epoch; */
	sh->o = 0;
	strcpy(sh->ko,"Origin");
}


/*
**  Convert integer data to floating point data and place in SAC output buffer
*/
int sac_conv(float *od,struct sac_hdr *sh,int *id,int np)
{
        register float *out;
        register int count = np;

        out = od + sh->npts;
        while( count-- ) *out++ = *id++;

        sh->npts += np;
        sh->e   += (float)(np-1) * sh->delta;
}

int flush_sac_file(struct sac_hdr *sh,float *sd)
{
        register int count;
        register float *dp;
        register float scale;
        register float minx;
        register float maxx;
        register double sum;
	struct date_time dt;
	int n_phases;	/* Number of phases to mark */
	int **tt_curves;
	char **phases;
	int i_count, f_count;	/* Initial and final counts */
	char string[10];	/* temporary place to hold lower case station name */

        /* make up a file name for the output */
        TRIM(sh->kstnm,8);
        TRIM(sh->kcmpnm,8);
	strcpy(string,sh->kstnm);
	lower(string);		/* change to lower case */
	if(sac_seq)
        	sprintf(file_name,"%s%s.%02d.%s%s",
			sac_prefix,string,sac_seq,sh->kcmpnm,sac_suffix);
	else
        	sprintf(file_name,"%s%s.%s%s",
			sac_prefix,string,sh->kcmpnm,sac_suffix);
        FPAD(file_name,30);
        FPAD(sh->kstnm,8);
        FPAD(sh->kcmpnm,8);

	/*******************************************************
	** ALJ: 8/22/90: Add in capability to pull out a portion of the
	** record.  To do this, we use the minimum and maximum times as
	** specified by the user or by the travel time curves.  If the
	** minimum time is less than sh->b, don't do anything.  If
	** minimum time is greater than sh->b, adjust accordingly.  Do
	** the same for the final time, sh->e.
	*******************************************************/
	
	i_count = 0;
	f_count = sh->npts;
	if(i_time > sh->b)
	{
		i_count = 1 + (i_time - sh->b)/sh->delta;
		if(i_count > sh->npts) return;
		sh->b += i_count*sh->delta;
	}
	if(f_time < sh->e)
	{
		f_count = sh->npts - 1 - (sh->e - f_time)/sh->delta;
		sh->e = sh->b + (f_count - i_count)*sh->delta;
	}
	if(sh->b > sh->e) return;

	/************************************
	** Adjust reference time to trace time if called for
	** B, E, O, and KZTIME
	************************************/
	if(o_trace) {
		minx = sh->b;
		sh->o -= minx;
		sh->e -= minx;
		sh->b = 0;
		dt.year = sh->nzyear;
		dt.doy = sh->nzjday;
		dt.hour = sh->nzhour;
		dt.minute = sh->nzmin;
		dt.second = sh->nzsec;
		dt.second += .001*sh->nzmsec;
		htoe(&dt);		/* compute epoch time */
		dt.epoch += minx;	/* add on time shift */
		etoh(&dt);		/* Go back to year, month, date format */
		sh->nzyear = dt.year;
		sh->nzjday = dt.doy;
		sh->nzhour = dt.hour;
		sh->nzmin = dt.minute;
		sh->nzsec = dt.second;
		sh->nzmsec = (int)(1000 * dt.second)%1000;
		strcpy(sh->ko,"Begin");
	}
	/*** Mark phases ***/
	tt_curves = phase_ttc;
	phases = mark_phases;
	n_phases = 0;
	while(*phases) {
		sh->t[n_phases] = sh->o+inter_ttc(*tt_curves,sh->gcarc);
		strcpy(sh->kt[n_phases],*phases);
		phases++;
		tt_curves++;
		n_phases++;
	}
	
	count = f_count - i_count + 1;
	sh->npts = count;
        dp = sd + i_count;
        scale = sh->scale;
        sh->scale = 1.0;
  	/*
	**  Since scale is not used in the header, we roll the scaling
        **  due to calib right into the data. This loop also calculates
        **  the max, min and mean while we are at it.
        */
	/*** 12/10/90: Take out scaling.  Let user do own scaling based on instr response ***/
        sum = 0.;
        minx = *dp;
        maxx = *dp;
        while( count-- ){
                /* *dp     *= scale; */
                if( *dp < minx ) minx = *dp;
                if( *dp > maxx ) maxx = *dp;
                sum     += *dp;
                dp++;
        }
        sh->depmin = minx;
        sh->depmax = maxx;
        sh->depmen = sum / sh->npts;

        cphdr0_();
        cphdr1_(&sh->npts,&sh->b,&sh->e,
                &sh->delta,&sh->scale,&sh->depmin,&sh->depmax,&sh->depmen);
        cphdr2_(&sh->nzyear,&sh->nzjday,
                &sh->nzhour,&sh->nzmin,&sh->nzsec,&sh->nzmsec);
        cphdr3_(sh->kstnm,&sh->stla,&sh->stlo,&sh->stel,8);
        cphdr4_(sh->kcmpnm,&sh->cmpaz,&sh->cmpinc,8);
        cphdr5_(&sh->evla,&sh->evlo,&sh->evdp,&sh->dist,&sh->az,&sh->baz,&sh->gcarc);
        cphdr6_(&sh->o,sh->ko,8);
	cphdr7_(&n_phases,
		&sh->t[0],&sh->t[1],&sh->t[2],&sh->t[3],&sh->t[4],
		&sh->t[5],&sh->t[6],&sh->t[7],&sh->t[8],&sh->t[9],
		sh->kt[0],sh->kt[1],sh->kt[2],sh->kt[3],sh->kt[4],
		sh->kt[5],sh->kt[6],sh->kt[7],sh->kt[8],sh->kt[9],
		8,8,8,8,8,8,8,8,8,8,8,8);

        /* now write the SAC file */
        sgluew_(file_name,sd + i_count,30);
	/*printf("SAC file %-30s\n",file_name);*/
	write_tty("SAC file ");
	write_tty(file_name);
	write_tty("\r\n");	/* Write into TTY window */	
}

/*
**  This routine just finds the first calibration constant for the correct
**  component that was in the table obtained from data log 1.
**  ( A lookup procedure based on time would be better )
*/
float
find_calib(int cn,struct data_log1 *dl)
{
        int i;
        for( i = 0 ; i < dl->ncals ; i++ )
                if( (cn+1) == dl->chanid[i] ) return( dl->calib[i] );
        return(0.0);
}


/*  sift routine for distance and time window  */
static int sift_rt(struct event_log *el,
	struct data_header *dh,
	struct station_log *sl,
	struct sac_hdr *sh,
	int nev,
	float rmin,
	float rmax,
	float *tlag)
{
	int nerr;
	int iok=1;
	int nsta=1;

	sh->evla = el->elat[nev-1];
	sh->evlo = el->elon[nev-1];
	distaz_(&sh->evla,&sh->evlo,&sl->lat,&sl->lon,&nsta,
               &sh->dist,&sh->az,&sh->baz,&sh->gcarc,&nerr);
	/*
	printf("\nStation %6s, Range %-7.2f\n",sl->sta,sh->gcarc);
	*/
	if (sh->gcarc < rmin || rmax < sh->gcarc) iok=0;
	if( !iok){
		return(iok);
	}

	/*** Subtract data header epoch time from event epoch time to get lag ***/
	*tlag = dh->dt.epoch - el->edt[nev-1].epoch;

	/*** 9/28/90: Was eliminating this record if lag is zero.  However, record can ***/
	/*** start before event so eliminate only if last data point is negative. ***/
	if(*tlag + dh->nsamp/(dh->nchan*dh->smprat) < 0) iok = 0;
	if(!iok) return iok;

	/**********************************************************************
	** Now use travel time curve to see if we still want this record 
	** Only look at final time to eliminate entire record.  Initial time will be used
	** to select where in record to begin.  Also, set global variables i_time and
	** f_time which will be used in flush_sac().
	** 9/5/90:  ttc are in 10ths of seconds.  Use linear interpolation.
	** 9/20/90: switch back to seconds.
	**********************************************************************/
	i_time = inter_ttc(initial_ttc,sh->gcarc);
	f_time = inter_ttc(final_ttc,sh->gcarc);

	if(*tlag > f_time ) iok=0;
	if(!iok) {
		return iok;
	}

	(sh+1)->evla = sh->evla;
	(sh+1)->evlo = sh->evlo;
	(sh+1)->dist = sh->dist;
	(sh+1)->az = sh->az;
	(sh+1)->baz = sh->baz;
	(sh+1)->gcarc = sh->gcarc;
	(sh+2)->evla = sh->evla;
	(sh+2)->evlo = sh->evlo;
	(sh+2)->dist = sh->dist;
	(sh+2)->az = sh->az;
	(sh+2)->baz = sh->baz;
	(sh+2)->gcarc = sh->gcarc;
	
	return(iok);
}


/*************************
** proc_evt()
**
** Sub-routine to process a *.evt file
** where filename is name of evt file
*************************/
proc_evt(filename,path)
char *filename;
char *path;
{
	char string[100];
	int i, j;
	/************************************************
	** Find file and display events
	************************************************/

	strcpy(prefix,filename);	/* e.g. "8501300" portion */
	strcpy(string,path);
	i = strlen(path);
	/* If user doesn't have '/' on end of path, put it on */
	if(i) {
		if(*(path + i - 1) != '/') strcat(string,"/");
	}
	strcat(string,filename);
	i = strlen(string);
	strcat(string,".evt");

	if( (ifd = open(string,O_RDONLY)) < 0 ){
		/* Try upper case ".EVT" */
		string[i] = 0;
		strcat(string,".EVT");
		if( (ifd = open(string,O_RDONLY)) < 0 ){
			printf("File %s not found\n",string);
			exit(1);
		}
	}
	{	

		/***********************************************
		** Open up CD-ROM file and start processing
		***********************************************/
		/*
		**  Put the first 3 blocks into prefix.hdr .
		*/
		strcpy(outfile,prefix);
		strcat(outfile,".hdr");
		if( (hfp = fopen(outfile,"w")) == NULL ){
			fprintf(stderr,"Can't open:%s\n",outfile);
			perror("evt_css");
			exit(3);
		}
		/*
		else printf("\nGenerating header file: %s\n",outfile);
		*/

		/*
		**  Open output file for record sifting parameters .
		*/
		/*
		**    Read and interpret the file header
		**       (not normally found on tapes)
		*/
		read_block(ifd,tblock);
		/* We have to read the first line to determine the CD-ROM version */
		sscanf(tblock+77,"%d",&cdver);
		if(cdver != 2)
		{
			cdver = 1;
			bwswap(tblock,2000);
		}
		line_split(tblock,cards);
		crack_file_header(cards,&file_header);
		for( i = 0 ; i < 25 ; i++ ) fprintf(hfp,"%s\n",cards[i]);

		/*
		**   Read and interpret the tape log
		*/
		/*** Read TAPE LOGS until get something else ***/
		/*** ALJ 90.09.14. Can't assume only one tape log ***/
		/*** loop until not a tape log ***/
		j = 0;	/* Block counter */
		while(0<read_block(ifd,tblock)) {
			line_split(tblock,cards);
			if(NULL == strstr(cards[0],"TAPE LOG")) break;
			crack_tape_log(cards,&tape_log,j++);
			for( i = 0 ; i < 25 ; i++ ) {
				fprintf(hfp,"%s\n",cards[i]);
			}
		}
		/*
		**   Read and interpret the event log
		*/
		/*	read_block(ifd,tblock); */
		/* line_split(tblock,cards);	*/
		crack_event_log(cards,&event_log);
		for( i = 0, j = 0 ; i < 25 ; i++ ) {
			fprintf(hfp,"%s\n",cards[i]);
		
		}
		/***************************
		** Call program which gets which event
		***************************/
		nev = which_evt(cards);
		fclose(hfp);
		write_sac();
	}
}
